/**
 * Copyright &copy; 2021-2026 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.sys.controller;


import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.google.gson.Gson;
import com.jeeplus.aop.demo.annotation.DemoMode;
import com.jeeplus.common.SecurityUtils;
import com.jeeplus.common.TokenProvider;
import com.jeeplus.common.constant.CacheNames;
import com.jeeplus.common.constant.CommonConstants;
import com.jeeplus.common.excel.ExcelOptions;
import com.jeeplus.common.excel.annotation.ExportMode;
import com.jeeplus.common.redis.RedisUtils;
import com.jeeplus.common.utils.ResponseUtil;
import com.jeeplus.core.query.QueryWrapperGenerator;
import com.jeeplus.logging.annotation.ApiLog;
import com.jeeplus.sys.domain.*;
import com.jeeplus.sys.domain.vo.UserVO;
import com.jeeplus.sys.feign.IUserApi;
import com.jeeplus.sys.mapper.UserMapper;
import com.jeeplus.sys.service.*;
import com.jeeplus.sys.service.dto.*;
import com.jeeplus.sys.service.mapstruct.CertWrapper;
import com.jeeplus.sys.service.mapstruct.UserWrapper;
import com.jeeplus.sys.utils.*;
import com.jeeplus.sys.utils.excel.UserEasyExcel;
import com.jeeplus.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 用户Controller
 *
 * @author jeeplus
 * @version 2021-8-29
 */
@Api(tags = "用户管理")
@RestController
@RequestMapping("/sys/user")
public class UserController {


    @Autowired
    private UserService userService;

    @Autowired
    private OtherUserInfoService otherUserInfoService;

    @Autowired
    private RosterBankCardsService rosterBankCardsService;
    @Autowired
    private RosterCertificateService rosterCertificateService;

    @Autowired
    private CertService certService;

    @Autowired
    private RosterContractService rosterContractService;

    @Autowired
    private OfficeService officeService;

    @Autowired
    private RedisUtils redisUtils;

    /**
     * 根据ids查询用户基本信息（姓名、手机、角色、部门）
     *
     * @param ids
     * @return
     */
    @ApiLog("根据id查询用户基本信息")
    @GetMapping("queryDetailList")
    @ApiOperation(value = "根据id查询用户基本信息")
    public ResponseEntity<List<UserDTO>> queryDetailList(@RequestParam("ids") String ids) {
        List<UserDTO> list = new ArrayList<>();
        String[] split = ids.split(",");
        for (String id : split) {
            UserDTO userDTO = userService.get(id);
            userDTO.setRoleList(userDTO.getRoleDTOList());
            list.add(userDTO);
        }
        return ResponseEntity.ok(list);
    }

    /**
     * 根据ids查询用户基本信息（姓名、手机、角色、部门）
     *
     * @param id
     * @return
     */
    @ApiLog("根据id查询用户基本信息")
    @GetMapping("queryDetailById")
    @ApiOperation(value = "根据id查询用户基本信息")
    public ResponseEntity<UserDTO> queryDetailById(@RequestParam("id") String id) {
        UserDTO userDTO = userService.get(id);
        userDTO.setRoleList(userDTO.getRoleDTOList());
        return ResponseEntity.ok(userDTO);
    }

    /**
     * 根据id查询用户
     *
     * @param id
     * @return
     */
    @ApiLog("查询用户")
    @GetMapping("queryById")
    @ApiOperation(value = "查询用户")
    public ResponseEntity queryById(@RequestParam("id") String id) {
        UserDTO userDTO = userService.get ( id );
        UserOtherInfo otherInfo = otherUserInfoService.getOne(new QueryWrapper<UserOtherInfo>().lambda().eq(UserOtherInfo::getUserInfoId, id));
        UserOtherInfoDto  otherInfoDto = new UserOtherInfoDto();
        if (null != otherInfo) {
            BeanUtils.copyProperties(otherInfo, otherInfoDto);
            userDTO.setOtherInfoDto(otherInfoDto);
        }

        //查询合同信息
        RosterContract contract = rosterContractService.getOne(new QueryWrapper<RosterContract>().lambda().eq(RosterContract::getBaseId, id));
        RosterContractDTO  contractDto = new RosterContractDTO();
        if (null != contract) {
            BeanUtils.copyProperties(contract, contractDto);
            userDTO.setRosterContractDTO(contractDto);
        }

        if (null != contract) {
            BeanUtils.copyProperties(contract, contractDto);
            userDTO.setRosterContractDTO(contractDto);
        }

        //查银行卡信息
        List<RosterBankCards> bankCards = rosterBankCardsService.list(new QueryWrapper<RosterBankCards>().lambda().eq(RosterBankCards::getBaseId, id));
        List<RosterBankCardsDTO>  bankCardsDTO = new ArrayList<>();
        bankCards.forEach(ba->{
            RosterBankCardsDTO cardsDTO = new RosterBankCardsDTO();
            BeanUtils.copyProperties(ba, cardsDTO);
            bankCardsDTO.add(cardsDTO);
        });
        userDTO.setRosterBankCardsDTOList(bankCardsDTO);

        //员工证书
        List<RosterCertificate> certificates = rosterCertificateService.list(new QueryWrapper<RosterCertificate>().lambda().eq(RosterCertificate::getBaseId, id));
        List<RosterCertificateDTO>  certificateDTOS = new ArrayList<>();
        certificates.forEach(ba->{
            RosterCertificateDTO cardsDTO = new RosterCertificateDTO();
            BeanUtils.copyProperties(ba, cardsDTO);
            certificateDTOS.add(cardsDTO);
        });
        userDTO.setRosterCertificateDTOList(certificateDTOS);

        //查询人员资质
        List<Cert> certs = certService.list(new QueryWrapper<Cert>().lambda().eq(Cert::getUserId, id));
        List<CertDTO>  certDTOS = new ArrayList<>();
        certs.forEach(ba->{
            CertDTO cardsDTO = new CertDTO();
            BeanUtils.copyProperties(ba, cardsDTO);
            certDTOS.add(cardsDTO);
        });
        userDTO.setCertDTOList(certDTOS);

        // 附件信息
        List<WorkAttachmentDto2> files = userService.getByAttachmentId(id);
        if (CollectionUtils.isNotEmpty(files)) {
            for (WorkAttachmentDto2 i : files) {
                i.setCreateBy(SpringUtil.getBean ( IUserApi.class ).getById(i.getBy()));
            }
        }
        userDTO.setFiles(files);
        return ResponseEntity.ok ( userDTO );
    }

    /**
     * 查询列表
     *
     * @param userDTO
     * @param page
     * @return
     */
    @ApiLog("项目组成员查询用户使用")
    @ApiOperation(value = "项目组成员查询用户使用")
    //@PreAuthorize("hasAuthority('sys:user:list')")
    @GetMapping("memberList")
    public ResponseEntity memberList(UserDTO userDTO, Page <UserDTO> page) throws Exception {

        QueryWrapper <UserDTO> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( userDTO, UserDTO.class );
        // 管理员查询不限制
        if (!UserUtils.getCurrentUserDTO().isAdmin()) {
            queryWrapper.isNull("a.is_admin");
            if (ObjectUtil.isNotEmpty(userDTO)){
                if (ObjectUtil.isNotEmpty(userDTO.getCompanyDTO()) && ObjectUtil.isNotEmpty(userDTO.getOfficeDTO())) {
                    OfficeDTO officeDTO = UserUtils.getCurrentUserDTO().getOfficeDTO();
                    if (StringUtils.isNotBlank(userDTO.getCertType())) {

                    }else{
                        // 当不是精确的部门查询时，再添加下面的筛选条件
                        if ((StringUtils.isNotBlank(userDTO.getCompanyDTO().getId()) && StringUtils.isBlank(userDTO.getOfficeDTO().getId()))
                                || "true".equals(userDTO.getSelectAll())) {
                            // officeDTO.id是空，companyDTO.id不是空的话，说明前端组件点击的集团来进行查询
                            // selectAll为‘true’的话，说明前端没有选择集团或者部门查询，这个时候要展示全部人员
                            // 根据当前用户的部门id找到此部门的ids
                            String ids = officeService.getById(UserUtils.getCurrentUserDTO().getOfficeDTO().getId()).getParentIds();
                            // 根据ids分隔后的index[2],找到当前用户的所属公司id
                            String companyId = null;
                            String[] split = ids.split(",");
                            // split.length大于2，说明当前登录人所属部门是普通部门，展示部分人员信息
                            // 否则，当前登录人所属公共类部门（忘了叫什么类部门了，就是总经办那种部门），展示全部人员信息
                            if(split.length>2){
                                companyId = split[2];
                                // 根据公司id,找到此公司下的所有部门
                                List<Office> officeList = officeService.list(new QueryWrapper<Office>().lambda().eq(Office::getParentId, companyId));
                                Set officeIdSet = new HashSet();
                                for (Office office : officeList) {
                                    if(StringUtils.isNotBlank(office.getId())){
                                        officeIdSet.add(office.getId());
                                    }
                                }
                                List<String> officeIdList = Lists.newArrayList(officeIdSet);
                                // 拿到这些部门的id
                                //List<String> officeIdList = officeList.stream().distinct().map(Office::getId).collect(Collectors.toList());
                                // 拿到所有公有部门的id
                                List<String> officePublicIdList = officeService.list(new LambdaQueryWrapper<Office>().eq(Office::getIsPublic,"1")).stream().map(Office::getId).collect(Collectors.toList());
                                // 拿到直属公司的id
                                officeIdList.add(companyId);
                                // 最后得到的所有部门的id
                                officeIdList.addAll(officePublicIdList);
                                List<String> lastIds = officeIdList.stream().distinct().collect(Collectors.toList());
                                // 添加筛选条件为这些部门的id
                                queryWrapper.in("a.office_id",lastIds);
                            }
                        } else if(StringUtils.isBlank(userDTO.getCompanyDTO().getId()) && StringUtils.isNotBlank(userDTO.getOfficeDTO().getId())) {
                            // officeDTO.id不是空，companyDTO.id是空的话，说明前端组件点击的部门查询
                            queryWrapper.eq("a.office_id",userDTO.getOfficeDTO().getId());
                        }
                    }
                }
            }
        }
        if (ObjectUtil.isNotEmpty(userDTO)) {
            // 当certType传值时，获取拥有此资格证类型的人员
            if (StringUtils.isNotBlank(userDTO.getCertType())) {
                queryWrapper.eq("sc.type",userDTO.getCertType());
            }
            if ( userDTO.getTenantDTO ( ) != null && StrUtil.isNotBlank ( userDTO.getTenantDTO ( ).getId ( ) ) ) {
            queryWrapper.eq ( "a.tenant_id", userDTO.getTenantDTO ( ).getId ( ) );
            }
        }
        IPage <UserDTO> result = userService.findMemberPage ( page, queryWrapper );
        return ResponseEntity.ok ( result );
    }

    /**
     * 查询列表
     *
     * @param userDTO
     * @param page
     * @return
     */
    @ApiLog("用户数据列表")
    @ApiOperation(value = "用户数据列表")
    //@PreAuthorize("hasAuthority('sys:user:list')")
    @GetMapping("list")
    public ResponseEntity list(UserDTO userDTO, Page <UserDTO> page) throws Exception {
//        QueryWrapper <UserDTO> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( userDTO, UserDTO.class );
//        if ( userDTO.getTenantDTO ( ) != null && StrUtil.isNotBlank ( userDTO.getTenantDTO ( ).getId ( ) ) ) {
//            queryWrapper.eq ( "a.tenant_id", userDTO.getTenantDTO ( ).getId ( ) );
//        }
//        IPage <UserDTO> result = userService.findPage ( page, queryWrapper );
//        return ResponseEntity.ok ( result );

        QueryWrapper <UserDTO> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( userDTO, UserDTO.class );
        // 管理员查询不限制
        if (!UserUtils.getCurrentUserDTO().isAdmin()) {
            queryWrapper.isNull("a.is_admin");
            if (ObjectUtil.isNotEmpty(userDTO)){
                if (ObjectUtil.isNotEmpty(userDTO.getCompanyDTO()) && ObjectUtil.isNotEmpty(userDTO.getOfficeDTO())) {
                    String id = UserUtils.getCurrentUserDTO().getCompanyDTO().getId();
                    if (StringUtils.isNotBlank(userDTO.getCertType())) {
                        //获取当前登陆人是否事中审公司人员
                        /*if(officeDTO.getParentIds().contains("1615171302381182978")){

                        }*/
                    }else{
                        // 当不是精确的部门查询时，再添加下面的筛选条件
                        if ((StringUtils.isNotBlank(userDTO.getCompanyDTO().getId()) && StringUtils.isBlank(userDTO.getOfficeDTO().getId()))
                                || "true".equals(userDTO.getSelectAll())) {
                            // officeDTO.id是空，companyDTO.id不是空的话，说明前端组件点击的集团来进行查询
                            // selectAll为‘true’的话，说明前端没有选择集团或者部门查询，这个时候要展示全部人员
                            // 根据当前用户的部门id找到此部门的ids
                            String ids = officeService.getById(UserUtils.getCurrentUserDTO().getOfficeDTO().getId()).getParentIds();
                            // 根据ids分隔后的index[2],找到当前用户的所属公司id
                            String companyId = null;
                            String[] split = ids.split(",");
                            // split.length大于2，说明当前登录人所属部门是普通部门，展示部分人员信息
                            // 否则，当前登录人所属公共类部门（忘了叫什么类部门了，就是总经办那种部门），展示全部人员信息
                            if(split.length>2){
                                companyId = split[2];
                                // 根据公司id,找到此公司下的所有部门
                                List<Office> officeList = officeService.list(new QueryWrapper<Office>().lambda().eq(Office::getParentId, companyId));
                                Set officeIdSet = new HashSet();
                                for (Office office : officeList) {
                                    if(StringUtils.isNotBlank(office.getId())){
                                        officeIdSet.add(office.getId());
                                    }
                                }
                                List<String> officeIdList = Lists.newArrayList(officeIdSet);
                                // 拿到这些部门的id
                                //List<String> officeIdList = officeList.stream().distinct().map(Office::getId).collect(Collectors.toList());
                                // 拿到所有公有部门的id
                                List<String> officePublicIdList = officeService.list(new LambdaQueryWrapper<Office>().eq(Office::getIsPublic,"1")).stream().map(Office::getId).collect(Collectors.toList());
                                // 拿到直属公司的id
                                officeIdList.add(companyId);
                                // 最后得到的所有部门的id
                                officeIdList.addAll(officePublicIdList);
                                List<String> lastIds = officeIdList.stream().distinct().collect(Collectors.toList());
                                // 添加筛选条件为这些部门的id
                                queryWrapper.in("a.office_id",lastIds);
                            }
                        } else if(StringUtils.isBlank(userDTO.getCompanyDTO().getId()) && StringUtils.isNotBlank(userDTO.getOfficeDTO().getId())) {
                            // officeDTO.id不是空，companyDTO.id是空的话，说明前端组件点击的部门查询
                            queryWrapper.eq("a.office_id",userDTO.getOfficeDTO().getId());
                        }
                    }
                }
            }
        }
        if (ObjectUtil.isNotEmpty(userDTO)) {
            // 当certType传值时，获取拥有此资格证类型的人员
            if (StringUtils.isNotBlank(userDTO.getCertType())) {

                //获取当前登录人信息
                UserDTO loginUserDTO = UserUtils.getCurrentUserDTO();
                //如果当前登录人是中审的管理咨询部人员，则注会列表中添加该部门选择项
                if(null != loginUserDTO && null != loginUserDTO.getOfficeDTO()){
                    OfficeDTO officeDTO = loginUserDTO.getOfficeDTO();
                    if(StringUtils.isNotBlank(officeDTO.getId())){
                        if("1602840084816138241".equals(officeDTO.getId())){
                            queryWrapper.and(
                                    wrapper -> wrapper.eq("sc.type",userDTO.getCertType()).or().eq("a.office_id","1602840084816138241")
                            );
                            //queryWrapper.eq("sc.type",userDTO.getCertType()).or().eq("a.office_id","1602840084816138241");
                        }else{
                            queryWrapper.eq("sc.type",userDTO.getCertType());
                        }
                    }else{
                        queryWrapper.eq("sc.type",userDTO.getCertType());
                    }
                }else{
                    queryWrapper.eq("sc.type",userDTO.getCertType());
                }

            }
            if ( userDTO.getTenantDTO ( ) != null && StrUtil.isNotBlank ( userDTO.getTenantDTO ( ).getId ( ) ) ) {
                queryWrapper.eq ( "a.tenant_id", userDTO.getTenantDTO ( ).getId ( ) );
            }
        }
        IPage <UserDTO> result = userService.findPage ( page, queryWrapper );
        IPage<UserDTO> result2 = userService.findPage3(userDTO);
        //判断 如果是签字注师筛选，则获取签字注师的被选择次数
        if(StringUtils.isNotBlank(userDTO.getCertType()) && "6".equals(userDTO.getCertType())){
            List<UserDTO> userList = result.getRecords();
            List<UserDTO> user2List = result2.getRecords();
            List<String> idList = Lists.newArrayList();
            for (UserDTO dto : user2List) {
                idList.add(dto.getId());
            }
            for (UserDTO info : userList) {
                if(idList.size()>0){
                    if(idList.contains(info.getId())){
                        info.setAccountantUserFlag(0);
                    }else{
                        info.setAccountantUserFlag(1);
                    }
                }else{
                    if("潘中".equals(info.getName())){
                        info.setAccountantUserFlag(0);
                    }else{
                        info.setAccountantUserFlag(1);
                    }
                }
                /*if("潘中".equals(info.getName())){
                    info.setAccountantUserFlag(0);
                }else{
                    info.setAccountantUserFlag(1);
                }*/
                Integer accountantUserCount = userService.getAccountantUserCount(info.getId());
                info.setAccountantUserCount(accountantUserCount);
            }
        }
        return ResponseEntity.ok ( result );
    }

    /**
     * 查询列表
     *
     * @param userDTO
     * @param page
     * @return
     */
    @ApiLog("用户数据列表")
    @ApiOperation(value = "用户数据列表")
    @GetMapping("allUserList")
    public ResponseEntity allUserList(UserDTO userDTO, Page <UserDTO> page) throws Exception {

        QueryWrapper <UserDTO> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( userDTO, UserDTO.class );
        // 管理员查询不限制
//        if (!UserUtils.getCurrentUserDTO().isAdmin()) {
//            if (ObjectUtil.isNotEmpty(userDTO)){
//                if (ObjectUtil.isNotEmpty(userDTO.getCompanyDTO()) && ObjectUtil.isNotEmpty(userDTO.getOfficeDTO())) {
//                    String id = UserUtils.getCurrentUserDTO().getCompanyDTO().getId();
//                    if (StringUtils.isNotBlank(userDTO.getCertType())) {
//                        //获取当前登陆人是否事中审公司人员
//                        /*if(officeDTO.getParentIds().contains("1615171302381182978")){
//
//                        }*/
//                    }else{
//                        // 当不是精确的部门查询时，再添加下面的筛选条件
//                        if ((StringUtils.isNotBlank(userDTO.getCompanyDTO().getId()) && StringUtils.isBlank(userDTO.getOfficeDTO().getId()))
//                                || "true".equals(userDTO.getSelectAll())) {
//                            // officeDTO.id是空，companyDTO.id不是空的话，说明前端组件点击的集团来进行查询
//                            // selectAll为‘true’的话，说明前端没有选择集团或者部门查询，这个时候要展示全部人员
//                            // 根据当前用户的部门id找到此部门的ids
//                            String ids = officeService.getById(UserUtils.getCurrentUserDTO().getOfficeDTO().getId()).getParentIds();
//                            // 根据ids分隔后的index[2],找到当前用户的所属公司id
//                            String companyId = null;
//                            String[] split = ids.split(",");
//                            // split.length大于2，说明当前登录人所属部门是普通部门，展示部分人员信息
//                            // 否则，当前登录人所属公共类部门（忘了叫什么类部门了，就是总经办那种部门），展示全部人员信息
//                            if(split.length>2){
//                                companyId = split[2];
//                                // 根据公司id,找到此公司下的所有部门
//                                List<Office> officeList = officeService.list(new QueryWrapper<Office>().lambda().eq(Office::getParentId, companyId));
//                                Set officeIdSet = new HashSet();
//                                for (Office office : officeList) {
//                                    if(StringUtils.isNotBlank(office.getId())){
//                                        officeIdSet.add(office.getId());
//                                    }
//                                }
//                                List<String> officeIdList = Lists.newArrayList(officeIdSet);
//                                // 拿到这些部门的id
//                                //List<String> officeIdList = officeList.stream().distinct().map(Office::getId).collect(Collectors.toList());
//                                // 拿到所有公有部门的id
//                                List<String> officePublicIdList = officeService.list(new LambdaQueryWrapper<Office>().eq(Office::getIsPublic,"1")).stream().map(Office::getId).collect(Collectors.toList());
//                                // 拿到直属公司的id
//                                officeIdList.add(companyId);
//                                // 最后得到的所有部门的id
//                                officeIdList.addAll(officePublicIdList);
//                                List<String> lastIds = officeIdList.stream().distinct().collect(Collectors.toList());
//                                // 添加筛选条件为这些部门的id
//                                queryWrapper.in("a.office_id",lastIds);
//                            }
//                        } else if(StringUtils.isBlank(userDTO.getCompanyDTO().getId()) && StringUtils.isNotBlank(userDTO.getOfficeDTO().getId())) {
//                            // officeDTO.id不是空，companyDTO.id是空的话，说明前端组件点击的部门查询
//                            queryWrapper.eq("a.office_id",userDTO.getOfficeDTO().getId());
//                        }
//                    }
//                }
//            }
//        }
        if (ObjectUtil.isNotEmpty(userDTO)) {
            // 当certType传值时，获取拥有此资格证类型的人员
            if (StringUtils.isNotBlank(userDTO.getCertType())) {

                //获取当前登录人信息
                UserDTO loginUserDTO = UserUtils.getCurrentUserDTO();
                //如果当前登录人是中审的管理咨询部人员，则注会列表中添加该部门选择项
                if(null != loginUserDTO && null != loginUserDTO.getOfficeDTO()){
                    OfficeDTO officeDTO = loginUserDTO.getOfficeDTO();
                    if(StringUtils.isNotBlank(officeDTO.getId())){
                        if("1602840084816138241".equals(officeDTO.getId())){
                            queryWrapper.and(
                                    wrapper -> wrapper.eq("sc.type",userDTO.getCertType()).or().eq("a.office_id","1602840084816138241")
                            );
                            //queryWrapper.eq("sc.type",userDTO.getCertType()).or().eq("a.office_id","1602840084816138241");
                        }else{
                            queryWrapper.eq("sc.type",userDTO.getCertType());
                        }
                    }else{
                        queryWrapper.eq("sc.type",userDTO.getCertType());
                    }
                }else{
                    queryWrapper.eq("sc.type",userDTO.getCertType());
                }

            }
            if ( userDTO.getTenantDTO ( ) != null && StrUtil.isNotBlank ( userDTO.getTenantDTO ( ).getId ( ) ) ) {
                queryWrapper.eq ( "a.tenant_id", userDTO.getTenantDTO ( ).getId ( ) );
            }
        }
        IPage <UserDTO> result = userService.findPage ( page, queryWrapper );
//        IPage<UserDTO> result2 = userService.findPage3(userDTO);
//        //判断 如果是签字注师筛选，则获取签字注师的被选择次数
//        if(StringUtils.isNotBlank(userDTO.getCertType()) && "6".equals(userDTO.getCertType())){
//            List<UserDTO> userList = result.getRecords();
//            List<UserDTO> user2List = result2.getRecords();
//            List<String> idList = Lists.newArrayList();
//            for (UserDTO dto : user2List) {
//                idList.add(dto.getId());
//            }
//            for (UserDTO info : userList) {
//                if(idList.size()>0){
//                    if(idList.contains(info.getId())){
//                        info.setAccountantUserFlag(0);
//                    }else{
//                        info.setAccountantUserFlag(1);
//                    }
//                }else{
//                    if("潘中".equals(info.getName())){
//                        info.setAccountantUserFlag(0);
//                    }else{
//                        info.setAccountantUserFlag(1);
//                    }
//                }
//                /*if("潘中".equals(info.getName())){
//                    info.setAccountantUserFlag(0);
//                }else{
//                    info.setAccountantUserFlag(1);
//                }*/
//                Integer accountantUserCount = userService.getAccountantUserCount(info.getId());
//                info.setAccountantUserCount(accountantUserCount);
//            }
//        }
        return ResponseEntity.ok ( result );
    }


    @ApiLog("根据用户id获取被选次数")
    @ApiOperation(value = "根据用户id获取被选次数")
    //@PreAuthorize("hasAuthority('sys:user:list')")
    @GetMapping("getCount")
    public ResponseEntity getCount(String id){
        UserDTO userDTO = userService.get(id);
        List<UserDTO> signatureAnnotator2List = userService.getSignatureAnnotator2List(new UserDTO());
        for (UserDTO dto : signatureAnnotator2List) {

            if (userDTO.getId().equals(dto.getId())){
                userDTO.setAccountantUserFlag(0);
                break;
            }else{
                userDTO.setAccountantUserFlag(1);
                // break;
            }
        }
        /*if (userDTO.getName().equals("潘中")){
            userDTO.setAccountantUserFlag(0);
        }else{
            userDTO.setAccountantUserFlag(1);
        }*/
        userDTO.setAccountantUserCount(userService.getAccountantUserCount(id));
        return ResponseEntity.ok(userDTO);
    }

    /**
     * 查询列表
     *
     * @param userDTO
     * @param page
     * @return
     */
    @ApiLog("用户数据列表")
    @ApiOperation(value = "用户数据列表")
    //@PreAuthorize("hasAuthority('sys:user:list')")
    @GetMapping("userList")
    public ResponseEntity userList(UserDTO userDTO, Page <UserDTO> page) throws Exception {

        QueryWrapper <UserDTO> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( userDTO, UserDTO.class );
        // 管理员查询不限制
        if (!UserUtils.getCurrentUserDTO().isAdmin()) {
            queryWrapper.isNull("a.is_admin");
            if (ObjectUtil.isNotEmpty(userDTO)){
                if (ObjectUtil.isNotEmpty(userDTO.getCompanyDTO()) && ObjectUtil.isNotEmpty(userDTO.getOfficeDTO())) {
                    //OfficeDTO officeDTO = UserUtils.getCurrentUserDTO().getOfficeDTO();
                    if (StringUtils.isNotBlank(userDTO.getCertType())) {
                        //获取当前登陆人是否事中审公司人员
                        /*if(officeDTO.getParentIds().contains("1615171302381182978")){

                        }*/
                    }else{
                        // 当不是精确的部门查询时，再添加下面的筛选条件
                        if ((StringUtils.isNotBlank(userDTO.getCompanyDTO().getId()) && StringUtils.isBlank(userDTO.getOfficeDTO().getId()))
                                || "true".equals(userDTO.getSelectAll())) {
                            // officeDTO.id是空，companyDTO.id不是空的话，说明前端组件点击的集团来进行查询
                            // selectAll为‘true’的话，说明前端没有选择集团或者部门查询，这个时候要展示全部人员
                            // 根据当前用户的部门id找到此部门的ids
                            Office byId = officeService.getById(UserUtils.getCurrentUserDTO().getOfficeDTO().getId());
                            if(null != byId){
                                String ids = officeService.getById(byId).getParentIds();
                                // 根据ids分隔后的index[2],找到当前用户的所属公司id
                                String companyId = null;
                                String[] split = ids.split(",");
                                // split.length大于2，说明当前登录人所属部门是普通部门，展示部分人员信息
                                // 否则，当前登录人所属公共类部门（忘了叫什么类部门了，就是总经办那种部门），展示全部人员信息
                                if(split.length>2){
                                    companyId = split[2];
                                    // 根据公司id,找到此公司下的所有部门
                                    List<Office> officeList = officeService.list(new QueryWrapper<Office>().lambda().eq(Office::getParentId, companyId));
                                    Set officeIdSet = new HashSet();
                                    for (Office office : officeList) {
                                        if(StringUtils.isNotBlank(office.getId())){
                                            officeIdSet.add(office.getId());
                                        }
                                    }
                                    List<String> officeIdList = Lists.newArrayList(officeIdSet);
                                    // 拿到这些部门的id
                                    //List<String> officeIdList = officeList.stream().distinct().map(Office::getId).collect(Collectors.toList());
                                    // 拿到所有公有部门的id
                                    List<String> officePublicIdList = officeService.list(new LambdaQueryWrapper<Office>().eq(Office::getIsPublic,"1")).stream().map(Office::getId).collect(Collectors.toList());
                                    // 拿到直属公司的id
                                    officeIdList.add(companyId);
                                    // 最后得到的所有部门的id
                                    officeIdList.addAll(officePublicIdList);
                                    List<String> lastIds = officeIdList.stream().distinct().collect(Collectors.toList());
                                    // 添加筛选条件为这些部门的id
                                    queryWrapper.in("a.office_id",lastIds);
                                }
                            }
                        } else if(StringUtils.isBlank(userDTO.getCompanyDTO().getId()) && StringUtils.isNotBlank(userDTO.getOfficeDTO().getId())) {
                            // officeDTO.id不是空，companyDTO.id是空的话，说明前端组件点击的部门查询
                            queryWrapper.eq("a.office_id",userDTO.getOfficeDTO().getId());
                        }
                    }
                }
            }
        }
        if (ObjectUtil.isNotEmpty(userDTO)) {
            // 当certType传值时，获取拥有此资格证类型的人员
            if (StringUtils.isNotBlank(userDTO.getCertType())) {
                queryWrapper.eq("sc.type",userDTO.getCertType());
            }
        }
        IPage <UserDTO> result = userService.userFindPage ( page, queryWrapper );
        return ResponseEntity.ok ( result );
    }

    @GetMapping("list2")
    public ResponseEntity list2(UserDTO userDTO, Page<UserDTO> page) throws Exception {
        if (ObjectUtil.isNotEmpty(userDTO.getOfficeDTO().getId())||ObjectUtil.isNotEmpty(userDTO.getLoginName())){
            IPage<UserDTO> result = userService.findPage3(userDTO);
            //判断 如果是签字注师筛选，则获取签字注师的被选择次数
            if (StringUtils.isNotBlank(userDTO.getCertType()) && "6".equals(userDTO.getCertType())) {
                List<UserDTO> userList = result.getRecords();
                for (UserDTO info : userList) {
                    //if ("潘中".equals(info.getName())) {
                        info.setAccountantUserFlag(0);
                    /*} else {
                        info.setAccountantUserFlag(1);
                    }*/
                    Integer accountantUserCount = userService.getAccountantUserCount(info.getId());
                    info.setAccountantUserCount(accountantUserCount);
                }
            }
            result.setTotal(result.getRecords().size());
            return ResponseEntity.ok(result);

        }else {
            IPage<UserDTO> result = userService.findPage1(userDTO,page);
            //判断 如果是签字注师筛选，则获取签字注师的被选择次数
            if (StringUtils.isNotBlank(userDTO.getCertType()) && "6".equals(userDTO.getCertType())) {
                List<UserDTO> userList = result.getRecords();
                for (UserDTO info : userList) {
//                    if ("潘中".equals(info.getName())) {
//                        info.setAccountantUserFlag(0);
//                    } else {
//                        info.setAccountantUserFlag(1);
//                    }
                    info.setAccountantUserFlag(0);
                    Integer accountantUserCount = userService.getAccountantUserCount(info.getId());
                    info.setAccountantUserCount(accountantUserCount);
                }
            }
            result.setTotal(result.getRecords().size());
            return ResponseEntity.ok(result);
        }
    }

//    /**
//     * 查询列表(排除admin)
//     *
//     * @param userDTO
//     * @param page
//     * @return
//     */
//    @ApiLog("用户数据列表")
//    @ApiOperation(value = "用户数据列表")
//    @PreAuthorize("hasAuthority('sys:user:list')")
//    @GetMapping("list2")
//    public ResponseEntity list2(UserDTO userDTO, Page <UserDTO> page) throws Exception {
//        QueryWrapper <UserDTO> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( userDTO, UserDTO.class );
//        if ( userDTO.getTenantDTO ( ) != null && StrUtil.isNotBlank ( userDTO.getTenantDTO ( ).getId ( ) ) ) {
//            queryWrapper.eq ( "a.tenant_id", userDTO.getTenantDTO ( ).getId ( ) );
//        }
//        IPage <UserDTO> result = userService.findPage2 ( page, queryWrapper );
//        return ResponseEntity.ok ( result );
//    }

    /**
     * 保存人员执业资质
     *
     * @param userDTO
     * @return
     */
    @DemoMode
    @ApiLog("保存人员执业资质")
    @PreAuthorize("hasAnyAuthority('sys:user:add', 'sys:user:edit')")
    @ApiOperation(value = "保存人员执业资质")
    @PostMapping("saveCertList")
    public ResponseEntity saveCertList(@Valid @RequestBody UserDTO userDTO) {
        userService.saveCertList(userDTO);
        return ResponseEntity.ok ( "保存成功!" );
    }

    /**
     * 保存用户
     *
     * @param userDTO
     * @return
     */
    @DemoMode
    @ApiLog("保存用户")
    @PreAuthorize("hasAnyAuthority('sys:user:add', 'sys:user:edit')")
    @ApiOperation(value = "保存用户")
    @PostMapping("save")
    public ResponseEntity save(@Valid @RequestBody UserDTO userDTO) {

        UserDTO info = SpringUtil.getBean ( IUserApi.class ).getByToken ( TokenProvider.getCurrentToken() );

        // 如果新密码为空，则不更换密码
        if ( StrUtil.isNotBlank ( userDTO.getNewPassword ( ) ) ) {
            userDTO.setPassword ( SecurityUtils.encryptPassword ( userDTO.getNewPassword ( ) ) );
        }
        if ( !UserUtils.isCheckLoginName ( userDTO.getOldLoginName ( ), userDTO.getLoginName ( ), userDTO.getTenantDTO ( ).getId ( ) ) ) {
            return ResponseEntity.badRequest ( ).body ( "保存用户'" + userDTO.getLoginName ( ) + "'失败，登录名已存在!" );
        }
        String id = SnowFlake2.getId();
        if (StringUtils.isBlank(userDTO.getId())) {
            userDTO.setId(id);
        }



        // 保存用户信息
        userService.saveOrUpdate ( userDTO );

        //保存合同信息
        if(ObjectUtil.isNotEmpty(userDTO.getRosterContractDTO())){
            RosterContractDTO rosterContractDTO = userDTO.getRosterContractDTO();

            RosterContract rosterContract = new RosterContract();
            BeanUtils.copyProperties(rosterContractDTO, rosterContract);

            rosterContract.setCreateById(info.getId());
            rosterContract.setCreateTime(new Date());
            rosterContract.setUpdateById(info.getId());
            rosterContract.setUpdateTime(new Date());

            rosterContract.setBaseId(userDTO.getId());
            rosterContractService.remove(new QueryWrapper<RosterContract>().eq("base_id",userDTO.getId()));
            rosterContractService.saveOrUpdate(rosterContract);
        } else {
            rosterContractService.remove(new QueryWrapper<RosterContract>().eq("base_id",userDTO.getId()));
        }
        //保存  银行卡信息、人员资质、员工证书  及  其他个人信息
        //1、保存其他个人信息
        if (null != userDTO.getOtherInfoDto()) {
            UserOtherInfoDto otherInfoDto = userDTO.getOtherInfoDto();
            UserOtherInfo otherInfo = new UserOtherInfo();
            BeanUtils.copyProperties(otherInfoDto, otherInfo);

            otherInfo.setCreateById(info.getId());
            otherInfo.setCreateTime(new Date());
            otherInfo.setUpdateById(info.getId());
            otherInfo.setUpdateTime(new Date());
            otherInfo.setUserInfoId(userDTO.getId());
            otherUserInfoService.saveOrUpdate(otherInfo);
        }
        //2、保存银行卡信息
        if(CollectionUtil.isNotEmpty(userDTO.getRosterBankCardsDTOList())){
            List<String> collect = userDTO.getRosterBankCardsDTOList().stream().map(RosterBankCardsDTO::getId).collect(Collectors.toList());
            rosterBankCardsService.remove(new QueryWrapper<RosterBankCards>().eq("base_id",userDTO.getId()).notIn(CollectionUtil.isNotEmpty(collect),"id",collect));
            userDTO.getRosterBankCardsDTOList().stream().forEach(item->{
                if(ObjectUtil.isNotEmpty(item)&& objectCheckIsNull(item)){
                    RosterBankCards rosterBankCards = new RosterBankCards();
                    BeanUtils.copyProperties(item, rosterBankCards);
                    rosterBankCards.setCreateById(info.getId());
                    rosterBankCards.setCreateTime(new Date());
                    rosterBankCards.setUpdateById(info.getId());
                    rosterBankCards.setUpdateTime(new Date());
                    rosterBankCards.setBaseId(userDTO.getId());
                    rosterBankCardsService.saveOrUpdate(rosterBankCards);
                }
            });
        }else{
            rosterBankCardsService.remove(new QueryWrapper<RosterBankCards>().eq("base_id",userDTO.getId()));
        }
        //3、保存员工证书
        //保存员工证书
        if(CollectionUtil.isNotEmpty(userDTO.getRosterCertificateDTOList())){
            List<String> collect = userDTO.getRosterCertificateDTOList().stream().map(RosterCertificateDTO::getId).collect(Collectors.toList());
            rosterCertificateService.remove(new QueryWrapper<RosterCertificate>().eq("base_id",userDTO.getId()).notIn(CollectionUtil.isNotEmpty(collect),"id",collect));
            userDTO.getRosterCertificateDTOList().stream().forEach(item->{
                if(ObjectUtil.isNotEmpty(item)&& objectCheckIsNull(item)){
                    RosterCertificate rosterCertificate = new RosterCertificate();
                    BeanUtils.copyProperties(item, rosterCertificate);
                    rosterCertificate.setCreateById(info.getId());
                    rosterCertificate.setCreateTime(new Date());
                    rosterCertificate.setUpdateById(info.getId());
                    rosterCertificate.setUpdateTime(new Date());
                    rosterCertificate.setBaseId(userDTO.getId());
                    rosterCertificate.setBaseId(userDTO.getId());
                    rosterCertificateService.saveOrUpdate(rosterCertificate);
                }
            });
        }else{
            rosterCertificateService.remove(new QueryWrapper<RosterCertificate>().eq("base_id",userDTO.getId()));
        }
        //4、保存人员资质
        // 保存花名册关联用户的资质信息
        certService.remove(new QueryWrapper<Cert>().lambda().eq(Cert::getUserId,userDTO.getId()));
        if(ObjectUtil.isNotEmpty(userDTO.getCertDTOList())){
            List<Cert> certList = new ArrayList<>();
            userDTO.getCertDTOList().stream().forEach(i -> {
                Cert cert = CertWrapper.INSTANCE.toEntity(i);
                cert.setUserId(userDTO.getId());
                cert.setId("");
                certList.add(cert);
            });
            if (CollectionUtil.isNotEmpty(certList)) {
                certService.saveBatch(certList);
            }
        }
        User user = userService.getOne(new QueryWrapper<User>().lambda().eq(User::getRosterId, userDTO.getId()));
        if (ObjectUtil.isNotEmpty(user)) {
            // 保存花名册关联用户的资质信息
//            certService.remove(new QueryWrapper<Cert>().lambda().eq(Cert::getUserId,user.getId()));
//            if(ObjectUtil.isNotEmpty(userDTO.getCertDTOList())){
//                List<Cert> certList = new ArrayList<>();
//                userDTO.getCertDTOList().stream().forEach(i -> {
//                    Cert cert = CertWrapper.INSTANCE.toEntity(i);
//                    cert.setUserId(user.getId());
//                    cert.setId("");
//                    certList.add(cert);
//                });
//                if (CollectionUtil.isNotEmpty(certList)) {
//                    certService.saveBatch(certList);
//                }
//            }
//            // 保存花名册关联用户的角色
//            userMapper.deleteUserRole(user.getId());  //  删除用户的角色
//            if (CollectionUtil.isNotEmpty(userDTO.getRoleIdList())) {
//                userDTO.getRoleIdList().stream().forEach(i -> {
//                    userMapper.insertUserRole(user.getId(), i);  // 插入用户的角色
//                });
//            }
//            // 保存花名册关联用户的岗位
//            userMapper.deleteUserPost(user.getId());  //  删除用户的岗位
//            if (CollectionUtil.isNotEmpty(userDTO.getPostIdList())) {
//                userDTO.getPostIdList().stream().forEach(i -> {
//                    userMapper.insertUserPost(user.getId(), i);  // 插入用户的岗位
//                });
//            }
//            // 保存花名册关联用户的管理部门
//            UserDTO u = new UserDTO();
//            u.setId(user.getId());
//            userMapper.deleteUserManageOffice(u);  //  删除用户的管理部门 （用户、管理部门关联表）
//            String oIds = "";
//            if (StringUtils.isNotBlank(userDTO.getManageOfficeIds())) {
//                List<String> officeIds= Arrays.asList(userDTO.getManageOfficeIds().split(","));
//                officeIds.stream().forEach(officeId -> {
//                    userMapper.insertUserManageOffice(officeId,user.getId());  // 插入用户的管理部门 （用户、管理部门关联表）
//                });
//                oIds = userDTO.getManageOfficeIds();
//            }
//            userMapper.updateUserManageOffice(oIds, user.getId()); // 修改用户的管理部门 （用户表）
        }
        //5、保存附件信息
        if (CollectionUtils.isNotEmpty(userDTO.getFiles())) {
            saveFiles(userDTO.getFiles(), userDTO, id);
        }

        return ResponseEntity.ok ( "保存用户'" + userDTO.getLoginName ( ) + "'成功!" );
    }

    /**
     * 用户信息显示编辑保存
     *
     * @param userDTO
     * @return
     */
    @DemoMode
    @ApiLog("修改个人资料")
    @ApiOperation(value = "修改个人资料")
    @PostMapping("saveInfo")
    public ResponseEntity saveInfo(@RequestBody UserDTO userDTO) {
        userService.updateById ( UserWrapper.INSTANCE.toEntity ( userDTO ) );
        UserUtils.deleteCache ( UserUtils.getCurrentUserDTO ( ) );
        return ResponseEntity.ok ( "修改个人资料成功!" );
    }

    /**
     * 批量删除用户
     */
    @DemoMode
    @ApiLog("删除用户")
    @ApiOperation(value = "删除用户")
    @PreAuthorize("hasAuthority('sys:user:del')")
    @DeleteMapping("delete")
    public ResponseEntity delete(String ids) {
        String idArray[] = ids.split ( "," );
        StringBuffer msg = new StringBuffer ( );
        boolean success = true;
        for (String id : idArray) {
            UserDTO userDTO = userService.get ( id );
            if ( UserUtils.getCurrentUserDTO ( ).getId ( ).equals ( userDTO.getId ( ) ) ) {
                success = false;
                msg.append ( "[" + userDTO.getLoginName ( ) + "]删除失败，不允许删除当前用户!<br/>" );
            } else if ( userDTO.isAdmin ( ) ) {
                success = false;
                msg.append ( "[" + userDTO.getLoginName ( ) + "]删除失败，不允许删除平台管理员!<br/>" );//删除用户失败, 不允许删除平台管理员用户
            } else {
                msg.append ( "[" + userDTO.getLoginName ( ) + "]删除成功!<br/>" );
                userService.deleteUser ( userDTO );//删除用户成功
            }
        }
        if ( success ) {
            return ResponseEntity.ok ( msg.toString ( ) );
        } else {
            return ResponseEntity.badRequest ( ).body ( msg.toString ( ) );
        }
    }

    /**
     * 导出用户数据
     *
     * @param userDTO
     * @param page
     * @param response
     * @throws Exception
     */
    @ApiLog("导出用户数据")
    @PreAuthorize("hasAnyAuthority('sys:user:export')")
    @GetMapping("export")
    @ApiOperation(value = "导出用户excel")
    public void exportFile(UserDTO userDTO, Page <UserDTO> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = options.getFilename ( );
        QueryWrapper <UserDTO> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( userDTO, UserDTO.class );
        if ( userDTO.getTenantDTO ( ) != null && StrUtil.isNotBlank ( userDTO.getTenantDTO ( ).getId ( ) ) ) {
            queryWrapper.eq ( "a.tenant_id", userDTO.getTenantDTO ( ).getId ( ) );
        }
        List <UserDTO> result;
        if ( ExportMode.current.equals ( options.getMode ( ) ) ) {
            result = userService.findPage ( page, queryWrapper ).getRecords ( );
        } else if ( ExportMode.selected.equals ( options.getMode ( ) ) ) {
            queryWrapper.in ( "a.id", options.getSelectIds ( ) );
            result = userService.findPage ( page, queryWrapper ).getRecords ( );
        } else {
            page.setSize ( -1 );
            page.setCurrent ( 0 );
            result = userService.findPage ( page, queryWrapper ).getRecords ( );
        }

        UserEasyExcel.newInstance ( ).exportExcel ( result, options.getSheetName ( ), UserDTO.class, fileName, options.getExportFields ( ), response );
    }

    /**
     * 导入用户数据
     *
     * @return
     */
    @DemoMode
    @PreAuthorize("hasAnyAuthority('sys:user:import')")
    @PostMapping("import")
    @ApiOperation(value = "导入用户excel")
    public ResponseEntity importFile(MultipartFile file) throws IOException {
        String result = UserEasyExcel.newInstance ( userService, UserWrapper.INSTANCE ).importExcel ( file, UserDTO.class );
        List<UserDTO> allUserInfo = userService.getAllUserInfo();
        if (redisUtils.hasKey("user:cache:user:all:info")){
            //先清除redis中的数据
            redisUtils.delete(CacheNames.USER_CACHE_USER_ALL_INFO,"user:cache:user:all:info");
            //再次将人员信息存入到redis中
            redisUtils.set(CacheNames.USER_CACHE_USER_ALL_INFO,"user:cache:user:all:info",allUserInfo);
        }else {
            //将人员信息存入到redis中
            redisUtils.set(CacheNames.USER_CACHE_USER_ALL_INFO,"user:cache:user:all:info",allUserInfo);
        }
        return ResponseEntity.ok ( result );
    }

    /**
     * 下载导入用户数据模板
     *
     * @param response
     * @return
     */
    @PreAuthorize("hasAnyAuthority('sys:user:import')")
    @GetMapping("import/template")
    @ApiOperation(value = "下载模板")
    public void importFileTemplate(HttpServletResponse response) throws IOException {
        String fileName = "用户数据导入模板.xlsx";
        List <UserDTO> list = Lists.newArrayList ( );
        list.add ( UserUtils.getCurrentUserDTO ( ) );
        UserEasyExcel.newInstance ( ).exportExcel ( list, "用户数据", UserDTO.class, fileName, null, response );
    }


    /**
     * 返回用户信息
     *
     * @return
     */
    @ApiLog("获取当前用户信息")
    @GetMapping("info")
    @ApiOperation(value = "获取当前用户信息")
    public ResponseEntity infoData() {
        return ResponseUtil.newInstance ( )
                .add ( "role", UserUtils.getRoleDTOList ( ) )
                .add ( "user", UserUtils.getCurrentUserDTO ( ) )
                .add ( "currentTenantId", TenantUtils.getTenantId ( ) )
                .ok ( "获取个人信息成功!" );
    }

    /**
     * 修改密码
     *
     * @param oldPassword
     * @param newPassword
     * @return
     */
    @DemoMode
    @ApiLog("修改密码")
    @PostMapping("/savePwd")
    public ResponseEntity savePwd(String oldPassword, String newPassword) {
        UserDTO userDTO = UserUtils.getCurrentUserDTO ( );
        if ( StrUtil.isNotBlank ( oldPassword ) && StrUtil.isNotBlank ( newPassword ) ) {
            if ( SecurityUtils.validatePassword ( oldPassword, userDTO.getPassword ( ) ) ) {
                User user = new User ( userDTO.getId ( ) );
                user.setPassword ( SecurityUtils.encryptPassword ( newPassword ) );
                user.setUpPassword("1");
                userService.updateById ( user );
                UserUtils.deleteCache ( userDTO );
                return ResponseEntity.ok ( "修改密码成功！" );
            } else {
                return ResponseEntity.badRequest ( ).body ( "修改密码失败，旧密码错误！" );
            }
        }
        return ResponseEntity.badRequest ( ).body ( "参数错误！" );
    }

    /**
     * 手机端-修改密码
     *
     * @return
     */
    @DemoMode
    @ApiLog("手机端-修改密码")
    @PostMapping("/savePwdByPhone")
    public ResponseEntity savePwdByPhone(@Valid @RequestBody UserVO userVO) {
        UserDTO userDTO = userService.get(userVO.getUserId());
        if ( StrUtil.isNotBlank ( userVO.getOldPassword() ) && StrUtil.isNotBlank ( userVO.getNewPassword() ) ) {
            if ( SecurityUtils.validatePassword ( userVO.getOldPassword(), userDTO.getPassword ( ) ) ) {
                User user = new User ( userDTO.getId ( ) );
                user.setPassword ( SecurityUtils.encryptPassword ( userVO.getNewPassword() ) );
                user.setUpPassword("1");
                userService.updateById ( user );
                UserUtils.deleteCache ( userDTO );
                return ResponseEntity.ok ( "修改密码成功！" );
            } else {
                return ResponseEntity.badRequest ( ).body ( "修改密码失败，旧密码错误！" );
            }
        }
        return ResponseEntity.badRequest ( ).body ( "参数错误！" );
    }

    /**
     * 获取菜单
     *
     * @return
     */
    @ApiLog("获取用户菜单")
    @GetMapping("getMenus")
    public ResponseEntity getMenus() {

//        return ResponseUtil.newInstance ()
//                .add ("dictList",  DictUtils.getDictMap()) //获取字典
//                .add("permissions", UserUtils.getPermissions())
//                .add ("menuList", MenuUtils.getMenuVOs () ).ok ();
        /**
         * 修改的原因，在某些性能很差的服务器上该方法的执行速度可以提升十几倍。当然在性能优良的服务器上，两种方式耗时几无区别。
         */
        Map map = new HashMap ( );
        map.put ( "dictList", DictUtils.getDictMap ( ) ); //获取字典
        map.put ( "permissions", UserUtils.getPermissions ( ) );
        map.put ( "menuList", MenuUtils.getMenuVOs ( ) );
        String res = new Gson ( ).toJson ( map );
        return ResponseEntity.ok ( res );
    }

    /**
     * 获取机构JSON数据。
     *
     * @param officeName 部门名称
     * @param extId 排除的ID
     * @param type  类型（1：公司；2：部门）
     * @param showAll 是否显示不可用数据 1 显示 0 隐藏
     * @return
     */
    @ApiLog("获取部门用户数据")
    @GetMapping("treeUserDataAllOffice")
    public ResponseEntity treeUserDataAllOffice(@RequestParam(required = false) String officeName,@RequestParam(required = false) String extId, @RequestParam(required = false) String type, @RequestParam(required = false, defaultValue = CommonConstants.NO) String showAll) {
        List<OfficeDTO> list = officeService.getOfficeAllByOfficeName (officeName);
        if(list.size()>0){
            List<String> officeIds = Lists.newArrayList();
            for (OfficeDTO info : list) {
                info.setTypeFlag(true);
                officeIds.add(info.getId());
            }
            //根据部门查询对应部门下的所有数据
            List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
            list.addAll(officeDTOList);
        }
        List rootTree = officeService.getRootTree (list, extId, type, showAll, "1");
        return ResponseEntity.ok (rootTree);
    }

    /**
     * 获取评估机构JSON数据。
     *
     * @param officeName 部门名称
     * @param extId 排除的ID
     * @param type  类型（1：公司；2：部门）
     * @param showAll 是否显示不可用数据 1 显示 0 隐藏
     * @return
     */
    @ApiLog("获取评估部门用户数据")
    @GetMapping("treeUserDataAccessOffice")
    public ResponseEntity treeUserDataAccessOffice(@RequestParam(required = false) String officeName,@RequestParam(required = false) String extId, @RequestParam(required = false) String type, @RequestParam(required = false, defaultValue = CommonConstants.NO) String showAll) {
        officeName = "评估部";
        List<OfficeDTO> list = officeService.getOfficeAllByOfficeName (officeName);
        if(list.size()>0){
            List<String> officeIds = Lists.newArrayList();
            for (OfficeDTO info : list) {
                info.setTypeFlag(true);
                officeIds.add(info.getId());
            }
            //根据部门查询对应部门下的所有数据
            List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
            list.addAll(officeDTOList);
        }
        List rootTree = officeService.getAccessRootTree (list, extId, type, showAll, "1");
        return ResponseEntity.ok (rootTree);
    }

    /**
     * 获取财务机构JSON数据。
     *
     * @param officeName 部门名称
     * @param extId 排除的ID
     * @param type  类型（1：公司；2：部门）
     * @param showAll 是否显示不可用数据 1 显示 0 隐藏
     * @return
     */
    @ApiLog("获取财务部门用户数据")
    @GetMapping("treeUserDataFinanceOffice")
    public ResponseEntity treeUserDataFinanceOffice(@RequestParam(required = false) String officeName,@RequestParam(required = false) String extId, @RequestParam(required = false) String type, @RequestParam(required = false, defaultValue = CommonConstants.NO) String showAll) {
        officeName = "兴光会计师事务所";
        List<OfficeDTO> list = officeService.getOfficeAllByOfficeName (officeName);
        if(list.size()>0){
            List<String> officeIds = Lists.newArrayList();
            for (OfficeDTO info : list) {
                info.setTypeFlag(true);
                officeIds.add(info.getId());
            }
            //根据部门查询对应部门下的所有数据
            List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
            list.addAll(officeDTOList);
        }
        List rootTree = officeService.getFinanceRootTree (list, extId, type, showAll, "1");
        return ResponseEntity.ok (rootTree);
    }

    /**
     * 判断当前用户是否是管理员
     *
     * @return
     */
    @ApiLog("判断当前用户是否是管理员")
    @GetMapping("isAdmin")
    @ApiOperation(value = "判断当前用户是否是管理员")
    public Boolean isAdmin() {
        return UserUtils.getCurrentUserDTO().isAdmin();
    }




    /**
     * 通过手机号获取验证码（忘记密码）
     * @param mobile 手机号码
     * @return
     */
    @ApiOperation(value = "通过手机号获取验证码（忘记密码）")
    @GetMapping(value = "/getPhoneCode")
    public ResponseEntity<HashMap<String,Object>> getPhoneCode(@RequestParam("mobile") String mobile) {
        HashMap<String,Object> j = new HashMap<String,Object>();
        //验证该手机号是否已经进行注册
        User user = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getMobile, mobile));
        if (ObjectUtil.isNotEmpty( user )) {
            //生成四位随机验证码
            String randomCode = String.valueOf((int) (Math.random() * 9000 + 1000));

            HashMap<String,Object> result = null;
            try{
                //调用工具类返回结果
                result = UserUtils.sendRandomCodes(mobile, randomCode);
                String statusCode = (String) result.get("statusCode");
                if (("000000").equals(statusCode)) {
                    j.put("success",true);
                    j.put("message","短信发送成功!");
                    //存放验证码
                    //以手机号+为key 五分钟为时效 将验证码进行短期存储
                    redisUtils.setEx(mobile+ "resetPassword", randomCode, 300, TimeUnit.SECONDS);
                }else if(statusCode.equals("160040")){
                    j.put("success",false);
                    j.put("message","手机号获取验证码次数已达每日上限!");
                }else{
                    j.put("success",false);
                    j.put("message","短信发送失败，错误代码：101，请联系管理员!");
                    j.put("ErrorXml",result);
                }
            }catch (Exception e){
                e.printStackTrace();
                j.put("success",false);
                j.put("message","短信发送失败!");
            }
        } else {
            j.put("success",false);
            j.put("message","当前手机号未注册!请联系管理员");
        }
        return ResponseEntity.ok(j);
    }

    /**
     * 修改密码
     * @param mobile
     * @param code
     * @param newPassword
     * @return
     */
    @GetMapping(value = "/saveNewPassword")
    @ApiOperation(value = "保存新密码")
    @Transactional(rollbackFor = Exception.class)
    public ResponseEntity<HashMap<String,Object>> saveNewPassword(@RequestParam("mobile") String mobile,
                                                                  @RequestParam("code") String code,
                                                                  @RequestParam("newPassword") String newPassword) {
        HashMap<String,Object> j = new HashMap<String,Object>();
        // 判断当前页面输入的验证码是否与redis中存储的验证码匹配
        String redisCode = (String) redisUtils.get(mobile + "resetPassword");
        if (org.apache.commons.lang3.StringUtils.isNotBlank(redisCode)) {
            if (redisCode.equals( code )) {
                // 进行密码修改操作
                User user = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getMobile, mobile));
                if (ObjectUtil.isNotEmpty( user )) {
                    if (org.apache.commons.lang3.StringUtils.isNotBlank( newPassword )) {
                        user.setPassword(SecurityUtils.encryptPassword ( newPassword ));
                        user.setUpPassword( "1" );
                        userService.updateById( user ); // 修改密码
                        redisUtils.delete(mobile + "resetPassword"); // 删除redis中的key
                        j.put("success",true);
                        j.put("message","密码修改成功!");
                    } else {
                        j.put("success",false);
                        j.put("message","请输入有效密码!");
                    }
                } else {
                    j.put("success",false);
                    j.put("message","当前手机号未注册!");
                }
            } else {
                j.put("success",false);
                j.put("message","验证码输入不正确!");
            }
        } else {
            j.put("success",false);
            j.put("message","当前手机验证码已失效，请重新发送验证码!");
        }
        return ResponseEntity.ok(j);
    }
    /**
     * 获取当前用户是否已经修改过密码
     *
     * @return
     */
    @ApiLog("获取当前用户是否已经修改过密码")
    @GetMapping("isUpdatePassword")
    @ApiOperation(value = "获取当前用户是否已经修改过密码")
    public ResponseEntity<Boolean> isUpdatePassword() {
        // 返回结果  true为以修改  false为未修改
        UserDTO userDTO = UserUtils.getCurrentUserDTO();
        Boolean updatePassword = userService.isUpdatePassword(userDTO.getId());
        return ResponseEntity.ok(updatePassword);
    }


    /**
     * 获取财务机构JSON数据(会计,总经办)。
     *
     * @param tenantId 部门id
     * @param extId 排除的ID
     * @param type  类型（1：公司；2：部门）
     * @param showAll 是否显示不可用数据 1 显示 0 隐藏
     * @return
     */
    @ApiLog("获取财务部门用户数据")
    @GetMapping("treeFinanceOffice")
    public ResponseEntity treeFinanceOffice(@RequestParam(required = false) String tenantId,@RequestParam(required = false) String extId, @RequestParam(required = false) String type, @RequestParam(required = false, defaultValue = CommonConstants.NO) String showAll) {
        UserDTO userDTO = UserUtils.getCurrentUserDTO();
        String id = userDTO.getTenantDTO().getId();
        if(id.equals("10005")){
            String officeName="兴光会计师事务所";
            List<OfficeDTO> list = officeService.getOfficeAllByOfficeName (officeName);
            if(list.size()>0){
                List<String> officeIds = Lists.newArrayList();
                for (OfficeDTO info : list) {
                    info.setTypeFlag(true);
                    officeIds.add(info.getId());
                }
                //根据部门查询对应部门下的所有数据
                List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
                list.addAll(officeDTOList);
            }
            String officeName2="兴光项目公司";
            List<OfficeDTO> list2 = officeService.getOfficeAllByOfficeName (officeName2);
            if(list2.size()>0){
                List<String> officeIds = Lists.newArrayList();
                for (OfficeDTO info : list2) {
                    info.setTypeFlag(true);
                    officeIds.add(info.getId());
                }
                //根据部门查询对应部门下的所有数据
                List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
                list2.addAll(officeDTOList);
            }
            List<OfficeDTO> objects1 = new ArrayList<>();
            objects1.addAll(list);
            objects1.addAll(list2);
            List rootTree = officeService.getFinanceTree (objects1, extId, type, showAll, "1");
            return ResponseEntity.ok (rootTree);
        }else if (id.equals("10002")){
            String officeName="兴光会计师事务所";
            List<OfficeDTO> list = officeService.getOfficeAllByOfficeName (officeName);
            if(list.size()>0){
                List<String> officeIds = Lists.newArrayList();
                for (OfficeDTO info : list) {
                    info.setTypeFlag(true);
                    officeIds.add(info.getId());
                }
                //根据部门查询对应部门下的所有数据
                List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
                list.addAll(officeDTOList);
            }
            String officeName1="综合管理公司";
            List<OfficeDTO> list1 = officeService.getOfficeAllByOfficeName (officeName1);
            if(list1.size()>0){
                List<String> officeIds = Lists.newArrayList();
                for (OfficeDTO info : list1) {
                    info.setTypeFlag(true);
                    officeIds.add(info.getId());
                }
                //根据部门查询对应部门下的所有数据
                List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
                list1.addAll(officeDTOList);
            }
            List<OfficeDTO> objects = new ArrayList<>();
            objects.addAll(list);
            objects.addAll(list1);
            List rootTree = officeService.getFinanceTree (objects, extId, type, showAll, "1");
            return ResponseEntity.ok (rootTree);

        }else if (id.equals("10006")){
            String officeName="兴光会计师事务所";
            List<OfficeDTO> list = officeService.getOfficeAllByOfficeName (officeName);
            if(list.size()>0){
                List<String> officeIds = Lists.newArrayList();
                for (OfficeDTO info : list) {
                    info.setTypeFlag(true);
                    officeIds.add(info.getId());
                }
                //根据部门查询对应部门下的所有数据
                List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
                list.addAll(officeDTOList);
            }
            String officeName1="苏州分公司";
            List<OfficeDTO> list1 = officeService.getOfficeAllByOfficeName (officeName1);
            if(list1.size()>0){
                List<String> officeIds = Lists.newArrayList();
                for (OfficeDTO info : list1) {
                    info.setTypeFlag(true);
                    officeIds.add(info.getId());
                }
                //根据部门查询对应部门下的所有数据
                List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
                list1.addAll(officeDTOList);
            }
            List<OfficeDTO> objects = new ArrayList<>();
            objects.addAll(list);
            objects.addAll(list1);
            List rootTree = officeService.getFinanceTree (objects, extId, type, showAll, "1");
            return ResponseEntity.ok (rootTree);

        }else if (id.equals("10004")){
            String officeName="兴光会计师事务所";
            List<OfficeDTO> list = officeService.getOfficeAllByOfficeName (officeName);
            if(list.size()>0){
                List<String> officeIds = Lists.newArrayList();
                for (OfficeDTO info : list) {
                    info.setTypeFlag(true);
                    officeIds.add(info.getId());
                }
                //根据部门查询对应部门下的所有数据
                List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
                list.addAll(officeDTOList);
            }
            String officeName1="中审江苏分所";
            List<OfficeDTO> list1 = officeService.getOfficeAllByOfficeName (officeName1);
            if(list1.size()>0){
                List<String> officeIds = Lists.newArrayList();
                for (OfficeDTO info : list1) {
                    info.setTypeFlag(true);
                    officeIds.add(info.getId());
                }
                //根据部门查询对应部门下的所有数据
                List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
                list1.addAll(officeDTOList);
            }
            List<OfficeDTO> objects = new ArrayList<>();
            objects.addAll(list);
            objects.addAll(list1);
            List rootTree = officeService.getFinanceTree (objects, extId, type, showAll, "1");
            return ResponseEntity.ok (rootTree);

        }else {
            String officeName="兴光会计师事务所";
            List<OfficeDTO> list = officeService.getOfficeAllByOfficeName (officeName);
            if(list.size()>0){
                List<String> officeIds = Lists.newArrayList();
                for (OfficeDTO info : list) {
                    info.setTypeFlag(true);
                    officeIds.add(info.getId());
                }
                //根据部门查询对应部门下的所有数据
                List<OfficeDTO> officeDTOList = userService.getUserByOffice(officeIds,type);
                list.addAll(officeDTOList);
            }
            List rootTree = officeService.getFinanceRootTree (list, extId, type, showAll, "1");
            return ResponseEntity.ok (rootTree);
        }
    }


    /**
     * 查询资质用户列表
     */
    @ApiLog("查询资质用户列表")
    @ApiOperation(value = "查询资质用户列表")
    //@PreAuthorize("hasAuthority('sys:user:list')")
    @GetMapping("certList")
    public ResponseEntity certList(UserDTO userDTO, Page <UserDTO> page) throws Exception {
        QueryWrapper <UserDTO> queryWrapper = QueryWrapperGenerator.buildQueryCondition ( userDTO, UserDTO.class );
        // 管理员查询不限制
        if (!UserUtils.getCurrentUserDTO().isAdmin()) {
            queryWrapper.isNull("a.is_admin");
            if (ObjectUtil.isNotEmpty(userDTO)){
                if (ObjectUtil.isNotEmpty(userDTO.getCompanyDTO()) && ObjectUtil.isNotEmpty(userDTO.getOfficeDTO())) {
                    String id = UserUtils.getCurrentUserDTO().getCompanyDTO().getId();
                    //判断当前登录人是否是综合管理公司人员
                    if (id.equals("1680778353809350658")){

                    }else {
                        //如果不是综合管理公司人员
                        queryWrapper.eq("a.company_id",id);
                    }
                    if (StringUtils.isNotBlank(userDTO.getCertType())) {
                        //获取当前登陆人是否事中审公司人员
                        /*if(officeDTO.getParentIds().contains("1615171302381182978")){

                        }*/
                    }else{
                        // 当不是精确的部门查询时，再添加下面的筛选条件
                        if ((StringUtils.isNotBlank(userDTO.getCompanyDTO().getId()) && StringUtils.isBlank(userDTO.getOfficeDTO().getId()))
                                || "true".equals(userDTO.getSelectAll())) {
                            // officeDTO.id是空，companyDTO.id不是空的话，说明前端组件点击的集团来进行查询
                            // selectAll为‘true’的话，说明前端没有选择集团或者部门查询，这个时候要展示全部人员
                            // 根据当前用户的部门id找到此部门的ids
                            String ids = officeService.getById(UserUtils.getCurrentUserDTO().getOfficeDTO().getId()).getParentIds();
                            // 根据ids分隔后的index[2],找到当前用户的所属公司id
                            String companyId = null;
                            String[] split = ids.split(",");
                            // split.length大于2，说明当前登录人所属部门是普通部门，展示部分人员信息
                            // 否则，当前登录人所属公共类部门（忘了叫什么类部门了，就是总经办那种部门），展示全部人员信息
                            if(split.length>2){
                                companyId = split[2];
                                // 根据公司id,找到此公司下的所有部门
                                List<Office> officeList = officeService.list(new QueryWrapper<Office>().lambda().eq(Office::getParentId, companyId));
                                Set officeIdSet = new HashSet();
                                for (Office office : officeList) {
                                    if(StringUtils.isNotBlank(office.getId())){
                                        officeIdSet.add(office.getId());
                                    }
                                }
                                List<String> officeIdList = Lists.newArrayList(officeIdSet);
                                // 拿到这些部门的id
                                //List<String> officeIdList = officeList.stream().distinct().map(Office::getId).collect(Collectors.toList());
                                // 拿到所有公有部门的id
                                List<String> officePublicIdList = officeService.list(new LambdaQueryWrapper<Office>().eq(Office::getIsPublic,"1")).stream().map(Office::getId).collect(Collectors.toList());
                                // 拿到直属公司的id
                                officeIdList.add(companyId);
                                // 最后得到的所有部门的id
                                officeIdList.addAll(officePublicIdList);
                                List<String> lastIds = officeIdList.stream().distinct().collect(Collectors.toList());
                                // 添加筛选条件为这些部门的id
                                queryWrapper.in("a.office_id",lastIds);
                            }
                        } else if(StringUtils.isBlank(userDTO.getCompanyDTO().getId()) && StringUtils.isNotBlank(userDTO.getOfficeDTO().getId())) {
                            // officeDTO.id不是空，companyDTO.id是空的话，说明前端组件点击的部门查询
                            queryWrapper.eq("a.office_id",userDTO.getOfficeDTO().getId());
                        }
                    }
                }
            }
        }
        if (ObjectUtil.isNotEmpty(userDTO)) {
            // 当certType传值时，获取拥有此资格证类型的人员
            if (StringUtils.isNotBlank(userDTO.getCertType())) {
                queryWrapper.eq("sc.type",userDTO.getCertType());
            }
            if ( userDTO.getTenantDTO ( ) != null && StrUtil.isNotBlank ( userDTO.getTenantDTO ( ).getId ( ) ) ) {
                queryWrapper.eq ( "a.tenant_id", userDTO.getTenantDTO ( ).getId ( ) );
            }
        }
        IPage <UserDTO> result = userService.findCertPage ( page, queryWrapper );
        //判断 如果是签字注师筛选，则获取签字注师的被选择次数
        if(StringUtils.isNotBlank(userDTO.getCertType()) && "6".equals(userDTO.getCertType())){
            List<UserDTO> userList = result.getRecords();
            for (UserDTO info : userList) {
                if("潘中".equals(info.getName())){
                    info.setAccountantUserFlag(0);
                }else{
                    info.setAccountantUserFlag(1);
                }
                Integer accountantUserCount = userService.getAccountantUserCount(info.getId());
                info.setAccountantUserCount(accountantUserCount);
            }
        }
        return ResponseEntity.ok ( result );
    }


    /**
     * 修改用户邮箱
     */
    @ApiLog("修改用户邮箱")
    @PostMapping("updateEmail")
    public String updateEmail(String id,String userEmail){
         userService.updateEmail(id,userEmail);
        return "修改成功";
    }

    /**
     * 根据用户id获取电话号码
     */
    @ApiLog("根据用户id获取电话号码")
    @PostMapping("getMobileById")
    public String getMobileById(String id){
        return userService.getMobileById(id);
    }



    public static boolean objectCheckIsNull(Object object) {
        boolean flag = false; //定义返回结果，默认为true

        if (Objects.isNull(object)) {
            flag = false;
        } else {
            Class clazz = (Class) object.getClass(); // 得到类对象
            Field fields[] = clazz.getDeclaredFields(); // 得到所有属性
            for (Field field : fields) {
                if("serialVersionUID".equals(field.getName()) || "BIZ_CODE".equalsIgnoreCase(field.getName())){
                    continue;
                }
                field.setAccessible(true);
                Object fieldValue = null;
                try {
                    fieldValue = field.get(object); //得到属性值
                    Type fieldType = field.getGenericType();//得到属性类型
                    String fieldName = field.getName(); // 得到属性名
                    System.out.println(("属性类型：" + fieldType + ",属性名：" + fieldName + ",属性值：" + fieldValue));
                } catch (IllegalArgumentException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
                if (fieldValue != null && fieldValue != "") {  //只要有一个属性值不为null 就返回false 表示对象不为null
                    flag = true;
                    break;
                }
            }
        }

        return flag;
    }

    public void saveFiles(List<WorkAttachmentDto2> list, UserDTO userDTO, String id) {
        int j = 1;
        for (WorkAttachmentDto2 dto : list) {
            WorkAttachmentInfo2 i = new WorkAttachmentInfo2();
            //包含了url、size、name
            i.setId(UUID.randomUUID().toString().replace("-", ""));
//            i.getCreateBy().setId(userDTO.getId());
            i.setCreateTime(new Date());
//            i.getUpdateBy().setId(userDTO.getId());
            i.setUpdateTime(new Date());
            i.setDelFlag(0);
            i.setUrl(dto.getUrl());
            //文件类型处理
            List<String> strings = Arrays.asList(dto.getName().split("\\."));
            if (org.flowable.editor.language.json.converter.util.CollectionUtils.isNotEmpty(strings)) {
                i.setType(strings.get(1));
            }
            i.setAttachmentId(id);
            i.setAttachmentName(dto.getName());
            i.setAttachmentFlag("userFilesInfo");
            i.setFileSize(dto.getSize());
            i.setSort(j);
            userService.insertWorkAttachment(i, userDTO);
            j++;
        }
    }
}
